/**
 * @file purple-io.h  Buffering I/O layer.
 * @ingroup core
 *
 * Pidgin is the legal property of its developers, whose names are too numerous
 * to list here.  Please refer to the COPYRIGHT file distributed with this
 * source distribution.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#ifndef _PURPLE_IO_H_
#define _PURPLE_IO_H_

#include "purple-fifo.h"
#include "sslconn.h"

G_BEGIN_DECLS

typedef struct PurpleIO PurpleIO;

typedef void (*PurpleIOFunc) (PurpleIO *io, gpointer data);

struct PurpleIO
{
	gint               fd;
	PurpleSslConnection *gsc;

	guint              got_eof : 1;

	PurpleFifo          *read_fifo;
	PurpleFifo          *write_fifo;

	guint              read_watch_id;
	guint              write_watch_id;

	/* Callback invoked when input buffer grows */
	PurpleIOFunc         read_func;
	gpointer           read_func_data;

	/* Callback invoked when output buffer shrinks (sufficiently) */
	PurpleIOFunc         write_func;
	gpointer           write_func_data;

	/* Callback invoked when connection drops */
	PurpleIOFunc         eof_func;
	gpointer           eof_func_data;

	/* Callback invoked when output buffer empty */
	PurpleIOFunc         flushed_func;
	gpointer           flushed_func_data;
};

PurpleIO            *purple_io_new                (gint fd);
PurpleIO            *purple_io_new_ssl            (PurpleSslConnection *gsc);
void               purple_io_destroy            (PurpleIO *io);

gint               purple_io_get_fd             (PurpleIO *io);
PurpleSslConnection *purple_io_get_ssl_connection (PurpleIO *io);
gboolean           purple_io_is_connected       (PurpleIO *io);

void               purple_io_set_read_func      (PurpleIO *io, PurpleIOFunc func, gpointer data);
void               purple_io_set_write_func     (PurpleIO *io, PurpleIOFunc func, gpointer data);
void               purple_io_set_eof_func       (PurpleIO *io, PurpleIOFunc func, gpointer data);
void               purple_io_set_flushed_func   (PurpleIO *io, PurpleIOFunc func, gpointer data);

gboolean           purple_io_write_bytes        (PurpleIO *io, gconstpointer libpurple, gint n_bytes);
gboolean           purple_io_write_line         (PurpleIO *io, const gchar *line);

gboolean           purple_io_read_bytes         (PurpleIO *io, gpointer dest, gint n_bytes);
gboolean           purple_io_read_line          (PurpleIO *io, gchar **line);

G_END_DECLS

#endif /* _PURPLE_IO_H_ */
